using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;

namespace domaincheck
{
    public partial class DomainForm : Form
    {
        string[] zones = { ".com", ".net", ".org", ".ru", ".info", ".biz", ".name", ".us" };
        Dictionary<string, bool> stats;
        List<string> twice;

        public DomainForm()
        {
            InitializeComponent();                     
        }

        private void OnCheck(object sender, EventArgs e)
        {
            progressBar.Maximum = zones.Length * 2;
            progressBar.Value = 0;
            string name = domainBox.Text;
            if (name.Length == 0) return;
            stats = new Dictionary<string, bool>();
            twice = new List<string>();
            btnCheck.Enabled = false;
            
            foreach (string zone in zones)
            {
                string host = name + zone;
                string whost = "www." + host;
                Dns.BeginGetHostAddresses(host, new AsyncCallback(HostCallback), host);
                Dns.BeginGetHostAddresses(whost, new AsyncCallback(HostCallback), whost);
            }            
        }

        void HostCallback(IAsyncResult ar)
        {
            bool found = false;
            try
            {
                IPAddress[] adrs = Dns.EndGetHostAddresses(ar);
                found = adrs != null && adrs.Length > 0;
            }
            catch (System.Net.Sockets.SocketException)
            {    }
            string host = (string)ar.AsyncState;

            lock (stats)
            {
                bool wasfound;
                if (stats.TryGetValue(host, out wasfound))
                {
                    stats.Remove(host);
                    stats.Add(host, wasfound || found);
                } else
                    stats.Add(host, found);                
            }

            if (!found && !twice.Contains(host))
            {
                twice.Add(host);
                System.Threading.Thread.Sleep(1000);
                Dns.BeginGetHostAddresses(host, new AsyncCallback(HostCallback), host);
            } else
                Invoke(new IncProgDel(IncreaseProgress));                
            
        }

        public delegate void IncProgDel();

        public void IncreaseProgress()
        {
            progressBar.Value++;
            ShowResult();
            if (progressBar.Value == zones.Length*2)
                btnCheck.Enabled = true;            
        }

        void ShowResult()
        {
            StringBuilder taken = new StringBuilder();
            StringBuilder free = new StringBuilder();
            lock (stats)
            {
                foreach (string host in stats.Keys)
                {
                    if (host.StartsWith("www.")) continue;
                    try
                    {
                        bool istaken = stats[host] || stats["www." + host];
                        (istaken ? taken : free).AppendFormat("{0} ", host);
                    }
                    catch (Exception) { }
                }
            }
            answerBox.Text = "Taken:\r\n" + taken.ToString() + "\r\n\r\nFree:\r\n" + free.ToString();            
        }        
    }
}